{*************************************************************}
{*                Unit FaDate  v1.0                          *}
{*    Copyright (c) 1998 by Dmitry Beloshistov [-=BDS=-]     *}
{*          Date & Time manipulation unit                    *}
{*************************************************************}

Unit FaDate;
Interface

{$IFDEF DPMI}
 {$DEFINE PMODE}
{$ENDIF}

{$IFDEF WINDOWS}
 {$DEFINE PMODE}
{$ENDIF}

{$IFNDEF PMODE}
 Uses Dos;
{$ELSE}
 Uses WinDos;
{$ENDIF}

type
     DateTimeRec=record
                  Day,Month,Year:Word;
                  Hour,Min,Sec  :Word;
                 end;
     dtFormat=(dtDateOnly,dtTimeOnly,dDateTimeRec);
     TDayNames=array[1..7] of String[20];
     TMonthNames=array[1..12] of String[20];
     TTimeNames=array[1..3] of String[20];

const DateDelimiter:Char='-';
      TimeDelimiter:Char=':';
      RusDaysNames:TDayNames=('ᥭ','쭨','୨',
                              '।','⢥','⭨','㡡');

      EngDaysNames:TDayNames=('Sunday', 'Monday', 'Tuesday', 'Wednesday',
                              'Thursday', 'Friday', 'Saturday');

      RusMonthNames:TMonthNames=('','ࠫ','','५',
                                 '','','','','',
                                 '','','');

      EngMonthNames:TMonthNames=('January', 'February', 'March', 'April',
                                 'May', 'June', 'July','August', 'September',
                                 'October', 'November', 'December');

      RusTimeNames:TTimeNames=('ᮢ','','ᥪ㭤');
      EngTimeNames:TTimeNames=('hours','mins','secs');

var DayNames:TDayNames;
    MonthNames:TMonthNames;
    TimeNames:TTimeNames;

Procedure ClrDateTime(var Dt:DateTimeRec);
{set all fields of DateTimeRec record to 0}
Procedure Date(var Dt:DateTimeRec);
{current date}
Procedure Time(var Dt:DateTimeRec);
{current time}
Procedure Now(var Dt:DateTimeRec);
{current timestamp}
Procedure EncodeDate(ADay,AMonth,AYear:Word; var Dt:DateTimeRec);
{convert Day,Month,Year to DateTimeRec}
Procedure EncodeTime(AHour,AMin,ASec:Word; var Dt:DateTimeRec);
{convert Hour,Min,Sec to DateTimeRec}
Procedure DecodeDate(Dt:DateTimeRec; var Day,Month,Year:Word);
{convert DateTimeRec to Day,Month,Year}
Procedure DecodeTime(Dt:DateTimeRec; var Hour,Min,Sec:Word);
{convert DateTimeRec to Hour,Min,Sec}
Function IsLeapYear(Year:Word):boolean;
{test for leap year}
Function DaysInMonth(Month,Year:Word):Word;
{days in month of year}
Function ValidDate(Dt:DateTimeRec):boolean;
{test for valid date}
function CurrentYear: Word;
{current year}
Function DayOfWeek(Dt:DateTimeRec):integer;
{day of week}
Function DateToLong(Dt:DateTimeRec):longint;
{convert DateTimeRec to longint (days after 01/01/0001)}
Function DayBetween(Dt1,Dt2:DateTimeRec):longint;
{days between two dates}
Procedure LongToDate(L:longint; var Dt:DateTimeRec);
{convert longint to DateTimeRec}
Procedure HowOldAsDate(WasBorn:DateTimeRec; var Old:DateTimeRec);
{how old as DateTimeRec}
Procedure HowOldAsDMY(WasBorn:DateTimeRec; var Days,Months,Years:Word);
{how old in Days,Months,Years}
Function DateToStr(Dt:DateTimeRec):String;
{convert date to string}
Function TimeToStr(Dt:DateTimeRec):String;
{convert time to string}
Function DateTimeToStr(Dt:DateTimeRec):String;
{convert date and time to string}
Function FormatDateTimeRec(Format:dtFormat; Dt:DateTimeRec):String;
{convert date and time to string with date/time format}
Procedure StrToDate(S:String; var Dt:DateTimeRec);
{convert string to date}
Procedure StrToTime(S:String; var Dt:DateTimeRec);
{convert string to time}
Procedure StrToDateTime(S:String; var Dt:DateTimeRec);
{convert string to timestamp}
Function LongDateStrL(Dt:DateTimeRec):String;
{long string of date}
Function LongTimeStrL(Dt:DateTimeRec):String;
{long string of time}
Function LongDateStr(Dt:DateTimeRec):String;
{long string of date}
Function LongTimeStr(Dt:DateTimeRec):String;
{long string of time}
Function DayName(Dt:DateTimeRec):String;
{name of day}

Implementation
Uses FaFuncs;

Procedure ClrDateTime(var Dt:DateTimeRec);
begin
  FillChar(Dt,SizeOf(DateTimeRec),0);
end;

Procedure Date(var Dt:DateTimeRec);
var Dw:Word;
begin
 ClrDateTime(Dt);
 with Dt do GetDate(Year,Month,Day,Dw);
end;

Procedure Time(var Dt:DateTimeRec);
var SS:Word;
begin
 ClrDateTime(Dt);
 with Dt do GetTime(Hour,Min,Sec,SS);
end;

Procedure Now(var Dt:DateTimeRec);
var SS:Word;
begin
 with Dt do
 begin
  GetDate(Year,Month,Day,SS);
  GetTime(Hour,Min,Sec,SS);
 end;
end;

Procedure EncodeDate(ADay,AMonth,AYear:Word; var Dt:DateTimeRec);
begin
 ClrDateTime(Dt);
 with Dt do
  begin Day:=ADay; Month:=AMonth; Year:=AYear; end;
end;

Procedure EncodeTime(AHour,AMin,ASec:Word; var Dt:DateTimeRec);
begin
 ClrDateTime(Dt);
 with Dt do
  begin Hour:=AHour; Min:=AMin; Sec:=ASec; end;
end;

Procedure DecodeDate(Dt:DateTimeRec; var Day,Month,Year:Word);
begin
 Day:=Dt.Day; Month:=Dt.Month; Year:=Dt.Year;
end;

Procedure DecodeTime(Dt:DateTimeRec; var Hour,Min,Sec:Word);
begin
 Hour:=Dt.Hour; Min:=Dt.Min; Sec:=Dt.Sec;
end;

Function IsLeapYear(Year:Word):boolean;
begin
 IsLeapYear:=(Year mod 4 = 0) and ((Year mod 100 <> 0) or (Year mod 400 = 0));
end;

Function DaysInMonth(Month,Year:Word):Word;
begin
 case Month of
  1,3,5,7,
  8,10,12 :DaysInMonth:=31;
  4,6,9,11:DaysInMonth:=30;
        2 :DaysInMonth:=28+Ord(IsLeapYear(Year));
  else DaysInMonth:=0;
 end;
end;

Function ValidDate(Dt:DateTimeRec):boolean;
begin
 with DT do
   case Month of
      1..12 :ValidDate:=Day<=DaysInMonth(Month,Year);
      else   ValidDate := False;
   end
end;

function CurrentYear: Word; assembler;
asm
  MOV     AH,2AH
  INT     21H
  MOV     AX,CX
end;

Function DayOfWeek(Dt:DateTimeRec):integer;
var OldDate:DateTimeRec;
    Dw:Word;
begin
 Now(OldDate);
 with Dt do
  begin
   SetDate(Year,Month,Day);
   GetDate(Year,Month,Day,Dw);
  end;
 with OldDate do SetDate(Year,Month,Day);
 DayOfWeek:=Dw;
end;

Function DateToLong(Dt:DateTimeRec):longint;
var i:word;
    L:longint;
begin
 L:=0;
 for i:=1 to Dt.Year do if IsLeapYear(i) then inc(L,366) else inc(L,365);
 for i:=1 to Pred(Dt.Month) do inc(L,DaysInMonth(i,Dt.Year));
 DateToLong:=L+Dt.Day;
end;

Function DayBetween(Dt1,Dt2:DateTimeRec):longint;
var L:longint;
begin
 L:=DateToLong(Dt1)-DateToLong(Dt2);
 if L<0 then DayBetween:=-L else DayBetween:=L;
end;

Procedure LongToDate(L:longint; var Dt:DateTimeRec);
var i:word;
    TestY:Word;
begin
 for i:=1 to $FFFF do
  begin
   if IsLeapYear(i) then TestY:=366 else TestY:=365;
   if L<TestY then break
   else begin Dec(L,TestY); Dt.Year:=i; end;
  end;
 for i:=1 to 12 do
  begin
   Dt.Month:=i;
   TestY:=DaysInMonth(i,Dt.Year);
   if L<TestY then break else Dec(L,TestY);
  end;
 Dt.Day:=Lo(L);
end;

Procedure HowOldAsDate(WasBorn:DateTimeRec; var Old:DateTimeRec);
var Dt:DateTimeRec;
    L1,L2:Longint;
begin
 Now(Dt);
 LongToDate(DayBetween(Dt,WasBorn),Old);
end;

Procedure HowOldAsDMY(WasBorn:DateTimeRec; var Days,Months,Years:Word);
var Old,Dt:DateTimeRec;
    L1,L2:Longint;
begin
 Now(Dt);
 LongToDate(DayBetween(Dt,WasBorn),Old);
 DecodeDate(Old,Days,Months,Years);
end;

Function DateToStr(Dt:DateTimeRec):String;
begin
 DateToStr:=lz(Dt.Day)+DateDelimiter+lz(Dt.Month)+DateDelimiter+IntToStr(DT.Year);
end;

Function TimeToStr(Dt:DateTimeRec):String;
begin
 with Dt do
  TimeToStr:=lz(Hour)+TimeDelimiter+lz(Min)+TimeDelimiter+lz(Sec);
end;

Function DateTimeToStr(Dt:DateTimeRec):String;
begin
 DateTimeToStr:=DateToStr(Dt)+' '+TimeToStr(Dt);
end;

Function FormatDateTimeRec(Format:dtFormat; Dt:DateTimeRec):String;
begin
 case Format of
  dtDateOnly:FormatDateTimeRec:=DateToStr(Dt);
  dtTimeOnly:FormatDateTimeRec:=TimeToStr(Dt);
  dDateTimeRec:FormatDateTimeRec:=DateTimeToStr(Dt);
 end;
end;

Procedure StrToDate(S:String; var Dt:DateTimeRec);
var Sbuf:String;
begin
 ClrDateTime(Dt);
 while S<>'' do
  begin
   SBuf:=GetFirstWord(S,[' ',DateDelimiter]);
   if Sbuf='' then break else Dt.Day:=StrToInt(SBuf);
   SBuf:=GetFirstWord(S,[' ',DateDelimiter]);
   if Sbuf='' then break else Dt.Month:=StrToInt(SBuf);
   SBuf:=GetFirstWord(S,[' ',DateDelimiter]);
   if Sbuf='' then break else Dt.Year:=StrToInt(SBuf);
  end;
end;

Procedure StrToTime(S:String; var Dt:DateTimeRec);
var Sbuf:String;
begin
 ClrDateTime(Dt);
 while S<>'' do
  begin
   SBuf:=GetFirstWord(S,[' ',TimeDelimiter]);
   if Sbuf='' then break else Dt.Hour:=StrToInt(SBuf);
   SBuf:=GetFirstWord(S,[' ',TimeDelimiter]);
   if Sbuf='' then break else Dt.Min:=StrToInt(SBuf);
   SBuf:=GetFirstWord(S,[' ',TimeDelimiter]);
   if Sbuf='' then break else Dt.Sec:=StrToInt(SBuf);
  end;
end;

Procedure StrToDateTime(S:String; var Dt:DateTimeRec);
var Sbuf:String;
begin
 ClrDateTime(Dt);
 while S<>'' do
  begin
   SBuf:=GetFirstWord(S,[' ',DateDelimiter]);
   if Sbuf='' then break else Dt.Day:=StrToInt(SBuf);
   SBuf:=GetFirstWord(S,[' ',DateDelimiter]);
   if Sbuf='' then break else Dt.Month:=StrToInt(SBuf);
   SBuf:=GetFirstWord(S,[' ',DateDelimiter]);
   if Sbuf='' then break else Dt.Year:=StrToInt(SBuf);
   SBuf:=GetFirstWord(S,[' ',TimeDelimiter]);
   if Sbuf='' then break else Dt.Hour:=StrToInt(SBuf);
   SBuf:=GetFirstWord(S,[' ',TimeDelimiter]);
   if Sbuf='' then break else Dt.Min:=StrToInt(SBuf);
   SBuf:=GetFirstWord(S,[' ',TimeDelimiter]);
   if Sbuf='' then break else Dt.Sec:=StrToInt(SBuf);
  end;
end;

Function LongDateStrL(Dt:DateTimeRec):String;
begin
 with Dt do LongDateStrL:=lz(Day)+' '+MonthNames[Month]+' '+IntToStr(Year);
end;

Function LongTimeStrL(Dt:DateTimeRec):String;
begin
 with Dt do LongTimeStrL:=lz(Hour)+' '+TimeNames[1]+' '+
                          lz(Min)+' '+TimeNames[1]+' '+
                          lz(Sec)+' '+TimeNames[3];
end;

Function LongDateStr(Dt:DateTimeRec):String;
begin
 with Dt do LongDateStr:=lz(Day)+' '+Copy(MonthNames[Month],1,3)+' '+IntToStr(Year);
end;

Function LongTimeStr(Dt:DateTimeRec):String;
begin
 with Dt do LongTimeStr:=lz(Hour)+' '+Copy(TimeNames[1],1,3)+' '+
                         lz(Min)+' '+Copy(TimeNames[1],1,3)+' '+
                         lz(Sec)+' '+Copy(TimeNames[3],1,3);
end;

Function DayName(Dt:DateTimeRec):String;
begin
 DayName:=DayNames[DayOfWeek(Dt)+1];
end;

Procedure IniDateTimeRecVariables;
begin
 DayNames:=RusDaysNames;
 MonthNames:=RusMonthNames;
 TimeNames:=RusTimeNames;
end;

begin
 IniDateTimeRecVariables;
end.