{*************************************************************}
{*                Unit ParList v1.0                          *}
{*    Copyright (c) 1998 by Dmitry Beloshistov [-=BDS=-]     *}
{*   Parameters list (can be use for config file managment)  *}
{*            Turbo/Borland Pascal version                   *}
{*************************************************************}
Unit ParList;

Interface
Uses Objects;

Type
    PLineCollection=^TLineCollection;
    TLineCollection=object(TStringCollection)
                     Procedure Change(Index:integer; NewValue:String);
                     Procedure Add(S:String);
                     Procedure AddStrings(SC:PStringCollection);
                     Procedure LoadFromFile(FileName:String);
                     Procedure SaveToFile(FileName:String);
                    end;

    PParamList=^TParamList;
    TParamList=object(TObject)
                 Params:PStringCollection;
                 Values:PStringCollection;
                 DeleteComments:boolean;
                 ParamEquChar:Char;
                 CmtChar:Char;
                 constructor Init(ADelCmt:boolean; ParEquChar,ACmtChar:Char);
                 Destructor Done; virtual;
                 Procedure AddStrings(SC:PStringCollection);
                 Procedure Add(S:String);
                 Procedure Delete(Index:integer);
                 Function GetParam(Index:integer):String;
                 Function GetValue(Index:integer):String;
                 Function Strs(Index:integer):String;
                 Function ValueByParam(ParamName:String):String;
                 Procedure SplitParamValue(S:String; var Param,Value:String);
                 Procedure LoadFromFile(FileName:String);
                 Procedure SaveToFile(FileName:String);
                 Function GetParamIndex(ParamName:String):integer;
                 Procedure SetParam(ParamName,NewValue:String);
                 Procedure DeleteParam(ParamName:String);
               end;

Implementation
Uses FaFuncs;

Procedure TLineCollection.Change(Index:integer; NewValue:String);
begin
 if Index>=Count then exit;
 AtFree(Index);
 AtInsert(Index,NewStr(NewValue));
end;

Procedure TLineCollection.Add(S:String);
begin
 if S='' then S:=' ';
 AtInsert(Count,NewStr(S));
end;

Procedure TLineCollection.AddStrings(SC:PStringCollection);
var i:integer;
begin
 for i:=0 to Pred(SC^.Count) do Add(PString(SC^.At(i))^);
end;

Procedure TLineCollection.LoadFromFile(FileName:String);
var S:String;
    F:Text;
begin
 FreeAll;
 Assign(F,FileName);
 {$I-} reset(F); {$I+}
 if IOResult<>0 then exit;
 while not eof(F) do
  begin Readln(F,S); Add(S); end;
 Close(F);
end;

Procedure TLineCollection.SaveToFile(FileName:String);
var i:integer;
    F:Text;
begin
 Assign(F,FileName);
 {$I-} rewrite(F); {$I+}
 for i:=0 to Pred(Count) do writeln(F,PString(At(i))^);
 Close(F);
end;

Constructor TParamList.Init(ADelCmt:boolean; ParEquChar,ACmtChar:Char);
begin
 Inherited Init;
 Params:=New(PStringCollection,Init(5,5));
 Values:=New(PStringCollection,Init(5,5));
 DeleteComments:=ADelCmt;
 ParamEquChar:=ParEquChar;
 CmtChar:=ACmtChar;
end;

Destructor TParamList.Done;
begin
 Dispose(Params,Done);
 Dispose(Values,Done);
 Inherited Done;
end;

Procedure TParamList.SplitParamValue(S:String; var Param,Value:String);
var N,C:integer;
begin
 if DeleteComments then S:=DelComment(S);
 N:=Pos(ParamEquChar,S); C:=Pos(CmtChar,S);
 if (C<>0) and (N>C) then N:=0;
 if N=0 then begin
               if C=0 then begin Param:=UpperCase(S); Value:=''; end
                else begin Param:=S; Value:=''; end;
             end
 else begin
       Param:=UpperCase(Trim(Copy(S,1,N-1)));
       Value:=Trim(Copy(S,N+1,Length(S)));
      end;
end;

Procedure TParamList.Add(S:String);
var P,V:String;
    OldCmtChar:Char;
begin
 if DeleteComments then
  begin OldCmtChar:=CommentChar; CommentChar:=CmtChar; end;
 SplitParamValue(S,P,V);
 if DeleteComments then CommentChar:=OldCmtChar;
 if P='' then exit;
 if V='' then V:=' ';
 Params^.AtInsert(Params^.Count,NewStr(P));
 Values^.AtInsert(Values^.Count,NewStr(V));
end;

Procedure TParamList.AddStrings(SC:PStringCollection);
var i:integer;
begin
 for i:=0 to Pred(SC^.Count) do Add(PString(SC^.At(I))^);
end;

Procedure TParamList.Delete(Index:integer);
begin
 Params^.AtFree(Index);
 Values^.AtFree(Index);
end;

Function TParamList.GetParam(Index:integer):String;
begin
 GetParam:=Trim(PString(Params^.At(Index))^);
end;

Function TParamList.GetValue(Index:integer):String;
begin
 GetValue:=Trim(PString(Values^.At(Index))^);
end;

Function TParamList.Strs(Index:integer):String;
var P,V:String;
begin
 V:=GetValue(Index);
 if V<>'' then Strs:=GetParam(Index)+ParamEquChar+V
 else Strs:=GetParam(Index);
end;

Function TParamList.GetParamIndex(ParamName:String):integer;
var Index:integer;
begin
 ParamName:=Trim(UpperCase(ParamName));
 GetParamIndex:=-1;
 for Index:=0 to Pred(Params^.Count) do
  if GetParam(Index)=ParamName then
    begin GetParamIndex:=Index; break; end;
end;

Function TParamList.ValueByParam(ParamName:String):String;
var Index:integer;
    OldCmt:Char;
begin
 ValueByParam:='';
 Index:=GetParamIndex(ParamName);
 if Index<>-1 then
   begin
    OldCmt:=CommentChar; CommentChar:=CmtChar;
    ValueByParam:=Trim(DelComment(GetValue(Index)));
    CommentChar:=OldCmt;
   end;
end;

Procedure TParamList.LoadFromFile(FileName:String);
var F:Text;
    S:String;
begin
 Assign(F,FileName);
 {$I-}
 Reset(F);
 {$I+}
 if IOResult<>0 then exit;
 Params^.FreeAll;
 Values^.FreeAll;
 while not eof(F) do begin Readln(F,S); Add(S); end;
 Close(F);
end;

Procedure TParamList.SaveToFile(FileName:String);
var F:Text;
    i:integer;
begin
 Assign(F,FileName);
 {$I-}
 Rewrite(F);
 {$I+}
 if IOResult<>0 then exit;
 for i:=0 to Pred(Params^.Count) do writeln(F,Strs(i));
 Close(F);
end;

Procedure TParamList.SetParam(ParamName,NewValue:String);
var Index:integer;
begin
 if Trim(ParamName)='' then exit;
 if Trim(NewValue)='' then NewValue:=' ';
 Index:=GetParamIndex(ParamName);
 if Index=-1 then Add(ParamName+ParamEquChar+NewValue)
 else begin
        Values^.AtFree(Index);
        Values^.AtInsert(Index,NewStr(NewValue));
      end;
end;

Procedure TParamList.DeleteParam(ParamName:String);
var Index:integer;
begin
 Index:=GetParamIndex(ParamName);
 if Index=-1 then exit;
 Delete(Index);
end;
{***********************************************************************}
{*                                DEMO                                 *}
{***********************************************************************}
(*-----------------------------------------------------------------------
Uses ParList;
var PL:PParamList;
begin
 PL:=New(PParamList,Init(False,'=',';'));
 PL^.LoadFromFile('fadmin.cfg');
 writeln(PL^.ValueByParam('TempFILEPaTH'));
 PL^.SetParam('TempFilePath','D:\Program Files\Temp Path');
 PL^.SetParam('ThisIsNewParam','This is New Param Value');
 PL^.SaveToFile('fadcfg.new');
 Dispose(PL,Done);
End.
-----------------------------------------------------------------------*)
end.